/*
****************************************************************************************************************************************************************
 Creating Random Data and Random Samples 
****************************************************************************************************************************************************************
Reference: Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire

Stata's random-number generation functions, such as "runiform()" and "rnormal()", do not really produce random numbers. 
These functions are deterministic algorithms that produce numbers that can pass for random. 

Stata in fact has ten random-number functions:

1)  runiform() generates rectangularly (uniformly) distributed random number over [0,1).
2)  rbeta(a,b) generates beta-distribution beta(a, b) random numbers.
3)  rbinomial(n,p) generates binomial(n, p) random numbers, where n is the number of trials and p the probability of a success.
4)  rchi2(df) generates ?2 with df degrees of freedom random numbers.
5)  rgamma(a,b) generates ?(a, b) random numbers, where a is the shape parameter and b, the scale parameter.
6)  rhypergeometric(N,K,n) generates hypergeometric random numbers, where N is the population size, 
	K is the number of in the population having the attribute of interest, and n is the sample size.
7)  rnbinomial(n,p) generates negative binomial  the number of failures before the nth success 
	 random numbers, where p is the probability of a success. (n can also be noninteger.)
8)  rnormal(?,?) generates Gaussian normal random numbers.
9)  rpoisson(m) generates Poisson(m) random numbers.
10) rt(df) generates Students t(df) random numbers.

Stata's random-number functions are formally called pseudorandom-number functions.
The sequences these functions produce are determined by the seed, which is just a number 
and which is set to 123456789 every time Stata is launched. 
This means that runiform() produces the same sequence each time you start Stata. 

To obtain different pseudorandom sequences from the pseudorandom-number functions, 
you must specify different seeds using the "set seed" command.

It does not really matter how you set the seed, as long as there is no obvious pattern in the seeds
that you set and as long as you do not set the seed too often during a session.

Your best choice for the seed is an element chosen randomly from the set {0,1,...,2,147,483,647}. 
****************************************************************************************************************************************************************

*** runiform()

The pseudo-random number function "runiform()" lies at the heart of Statas ability to generate random data 
or to sample randomly from the data at hand. 
If we presently have data in memory, then a command such as the following creates a new variable
named randnum, having apparently random 16-digit values over the interval [0,1). 

	use Nations2.dta, clear
	generate randnum = runiform()

We could also create a random dataset from scratch. To do so we first clear other data from memory.
Next, set the number of observations desired for the new dataset. 
Explicitly setting the seed number makes it possible to later reproduce the same "random" results. 
Finally, we generate our random variable. 

The following commands create a dataset with 10 observations and one variable, called randnum. */

clear
set obs 10
set seed 12345
generate randnum = runiform()
list

/* In combination with Stata's algebraic, statistical and special functions, 
runiform() can simulate values sampled from a variety of theoretical distributions. 
If we want newvar sampled from a uniform distribution over [0,428) instead of the usual [0,1), we type */

generate newvar = 428 * runiform()

/* These will still be 16-digit values. Perhaps we want only integers from 1 to 428 (inclusive). 
The ceiling or ceil() function provides a simple way to do this */

generate newvarceil = ceil(428 * runiform())

// To simulate 1,000 throws of a six-sided die, type

clear
set obs 1000
generate roll = ceil(6 * runiform())
tabulate roll

/* We theoretically expect 16.67% ones, 16.67% twos and so on, but in any one sample like these
1,000 throws, the observed percentages will vary randomly around their expected values. */

// To simulate 1,000 throws of a pair of six-sided dice, type

generate dice = ceil(6 * runiform()) + ceil(6 * runiform())
tabulate dice

* We can use _n to begin an artificial dataset as well. 
// The following commands create a new 5,000-observation dataset with one variable named index, containing values from 1 to 5,000.

clear
set obs 5000
generate index = _n
summarize
****************************************************************************************************************************************************************

*** rnormal()

/* To generate random variables from a normal (Gaussian) distribution, use the function "rnormal()". 
The following example creates a dataset with 2,000 observations and 2 variables:
z from an N(0,1) population, and u from N(500,75). */

clear
set obs 2000
generate z = rnormal()
generate u = rnormal(500,75)

// The actual sample means and standard deviations differ slightly from their theoretical values:

summarize

/* If z follows a normal distribution, v = e^z follows a lognormal distribution. 
To form a lognormal variable v based upon a standard normal distribution, */

generate v = exp(rnormal())

/* Taking logarithms, of course, normalizes a lognormal variable.
To simulate w values drawn randomly from an exponential distribution with mean and standard deviation equal to 3, */

generate w = -3 * ln(runiform())

/* For other means and standard deviations, substitute other values for 3.
x5 follows a chi-squared distribution with five degrees of freedom: */

generate x5 = rchi2(5)

// y follows a binomial distribution, given 10 trials and success probability .2:

generate y = rbinomial(10,.2)

// t45 follows a Students t distribution with 45 degrees of freedom:

generate t45 = rt(45)

// Type help random to see a list of other available functions for generating random variables
// from beta, gamma, hypergeometric, negative binomial or Poisson distributions.
****************************************************************************************************************************************************************

*** drawnorm

/* The drawnorm command provides an alternative way to generate multiple normal variables,
and optionally to specify the correlations between them. 

Using drawnorm to generate 5,000 observations of just one variable from N(0,1), type */

clear
drawnorm z, n(5000)
summarize

/* Below, we will create three further variables. 
Variable x1 is from an N(0,1) population, 
variable x2 is from N(100,15), 
and x3 is from N(500,75). 
Furthermore, we define these variables to have the following population correlations:

	x1		x2		x3
x1	1		0.4		-0.8
x2	0.4		1		0
x3	-0.8	0		1

The procedure for creating such data requires first defining the correlation matrix C, 
and then using C in the drawnorm command: */

mat C = (1, .4, -.8 \ .4, 1, 0 \ -.8, 0, 1)
drawnorm x1 x2 x3, means(0,100,500) sds(1,15,75) corr(C)
summarize x1-x3

correlate x1-x3

/* Compare the sample variables correlations and means with the theoretical values given earlier.
Random data generated in this fashion can be viewed as samples drawn from theoretical populations. 
We should not expect the samples to have exactly the theoretical population parameters. 

Artificial uncorrelated or correlated datasets also can be created via menus and dialog boxes, under

Statistics > Other > Draw a sample from a normal distribution
or
Statistics > Other > Create a dataset with specified correlation structure
****************************************************************************************************************************************************************

*** sample

The command "sample" makes unobtrusive use of runiform's random generator to obtain random samples of the data in memory. 

For example, to discard all but a 10% random sample of the original data, type

sample 10

// When we add an in or if qualifier, sample applies only to those observations meeting our criteria. For example,

sample 10 if age < 26

// would leave us with a 10% sample of those observations with age less than 26, plus 100% of the original observations with age >= 26.

// We could also select random samples of a particular size. To discard all but 90 randomly-selected observations from the dataset in memory, type

sample 90, count
****************************************************************************************************************************************************************

*/
